-- -------------------------------------------------------------------------- --
-- EnhancedStackSplit by kunda                                                --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- Features:                                                                  --
-- - 1-Click mode:                                                            --
--   -> 'M' Button (toggle: '1-Click mode' / 'Original WoW mode' [DEFAULT])   --
--   Split a stack with just one click when you click on one of the numbered  --
--   buttons (OK(1),2,3,4,5,6,7,8,9,10 and 20).                               --
--   Split is placed on the cursor!!! so you can place it in a slot, trade,   --
--   destroy or what ever.                                                    --
--                                                                            --
-- - Auto Split mode:                                                         --
--   -> 'Auto' Button                                                         --
--   Automatically splits a stack and place the splitted item(s) in a free    --
--   bag slot when you click on one of the numbered buttons (1,2,3,4,5,6,7,8, --
--   9,10 and 20). Free bag slots depends on ItemFamily from item you wanna   --
--   split and on ItemFamily from bag(s) with free slots. Items are placed    --
--   from top left to bottom right slot in a bag.                             --
--   example: Free bagslots = 20                                              --
--            Item with StackSize = 137                                       --
--            - you click on 5                                                --
--              you get: 20 items with 5 Itemstacksize in your bag(s)         --
--                       leftover = 37                                        --
--                       (20 because free bagslots = 20)                      --
--            - you click on 10                                               --
--              you get: 13 items with 10 Itemstacksize in your bag(s)        --
--                       leftover = 7                                         --
--            leftover and last n-stacksize remains in the original bagslot   --
--   Note: Auto-splitted items are only placed in the five normal bags        --
--         including special bags (or less if you have not five bags).        --
--         The 'Auto Split mode' should be only accessible via bags and bank, --
--         but I did not find a good way to do this. So it's possible that    --
--         this mode is available where it shouldn't be. If you try to auto   --
--         split in such cases the SplitFrame can stuck. Just press ESC to    --
--         close it. Bag and Bank Addons MUST provide the bag/slot IDs in the --
--         same way the original UI does, otherwise EnhancedStackSplit can    --
--         not auto-split such stacks or you can get the stuck-error I        --
--         described above.                                                   --
--         Most bag/bank addons works perfect with EnhancedStackSplit because --
--         the addon authors made a good job :)                               --
--                                                                            --
-- - Options:                                                                 --
--   Options (ESC) -> Interface -> AddOns Tab -> EnhancedStackSplit           --
--   - General:                                                               --
--     - Chat Announce [OFF by DEFAULT]                                       --
--       (Shows a message when the split mode is changed.)                    --
--                                                                            --
-- - Slash commands:                                                          --
--   /ess or /enhancedstacksplit -> Interface Options Menu                    --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- Credits:                                                                   --
-- LockSlot functions from ClosetGnome by Rabbit.                             --
--                                                                            --
-- Note:                                                                      --
-- You can shift-click on any stackable item, press a number on your keyboard --
-- and click OK or press ENTER to split a stack!!!                            --
--                                                                            --
-- -------------------------------------------------------------------------- --

EnhancedStackSplit_Options = {}                 -- SavedVariable table

local EnhancedStackSplit = CreateFrame("Frame") -- event container
local L = EnhancedStackSplit_Locales            -- localization table

local _G = _G
local autoSplitMode = false
local splitItemBag
local splitItemSlot
local maxStackSize = 0
local slotLocks = nil
local autoSplitStackCount
local autoSplitNumStacks
local autoSplitNumber
local autoSplitLeftOver
local autoSplitItemLink
local splitMode = {
	[1] = L["Original WoW mode"],
	[2] = L["1-Click mode"],
	[3] = L["Auto Split mode"]
}
local blockedFrames = {
	[1] = "MerchantItem",
	[2] = "GuildBank"
}
local mainBankFrames = {
	[1] = "BankFrame", -- default Blizzard Bank Frame
	[2] = "TBnkFrame"  -- TBag
}

-- Helper functions START ------------------------------------------------------
local function reclaim(t) -- LockSlot function
	if type(t) ~= "table" then return end
	for k, v in pairs(t) do
		if type(v) == "table" then
			t[k] = reclaim(v)
		else
			t[k] = nil
		end
	end
	t = nil
	return nil
end

local function LockSlot(bag, slot) -- LockSlot function
	if not slotLocks then slotLocks = {} end
	if not slotLocks[bag] then slotLocks[bag] = {} end
	slotLocks[bag][slot] = true
end

local function IsSlotLocked(bag, slot) -- LockSlot function
	if slotLocks and slotLocks[bag] and slotLocks[bag][slot] then
		return true
	end
	return false
end

local function ClearSlotLocks() -- LockSlot function
	slotLocks = reclaim(slotLocks)
end

local function Print(text)
	if EnhancedStackSplit_Options.chat then
		print("|cffffff7fEnhancedStackSplit: |r"..text)
	end
end

local function IsBag(bag)
	local invID = ContainerIDToInventoryID(bag)
	local bagLink = GetInventoryItemLink("player", invID)
	if bagLink then
		return bagLink
	else
		return false
	end
end

local function GetFreeBagSlots()
	local freeSlots = GetContainerNumFreeSlots(0)
	local containerBagType = 0
	local itemBagType = GetItemFamily(autoSplitItemLink)
	for bag = 1, NUM_BAG_SLOTS do
		local bagLink = IsBag(bag)
		if bagLink then
			containerBagType = GetItemFamily(bagLink)
			if containerBagType == 0 or containerBagType == itemBagType then
				freeSlots = freeSlots + GetContainerNumFreeSlots(bag)
			end
		end
	end
	return freeSlots
end

local function GetFreeSlot()
	local containerBagType = 0
	local itemBagType = GetItemFamily(autoSplitItemLink)
	local goodBag = true
	for bag = 0, NUM_BAG_SLOTS do
		if bag > 0 then
			local bagLink = IsBag(bag)
			if bagLink then
				containerBagType = GetItemFamily(bagLink)
				if containerBagType == 0 or containerBagType == itemBagType then
					goodBag = true
				else
					goodBag = false
				end
			end
		end
		if goodBag then
			for slot = 1, GetContainerNumSlots(bag) do
				if not IsSlotLocked(bag, slot) then
					if not GetContainerItemLink(bag, slot) then
						return bag, slot
					end
				end
			end
		end
	end
	return nil
end

local function AutoSplitCalc(num)
	local freeSlots = GetFreeBagSlots()
	local numStacks = floor(maxStackSize/num)
	if numStacks > freeSlots then
		numStacks = freeSlots
	end
	local leftOver = maxStackSize-(numStacks*num)
	return freeSlots or 0, numStacks or 0, leftOver or 0
end

local function SlashHandler()
	InterfaceOptionsFrame_OpenToCategory("EnhancedStackSplit")
end
-- Helper functions END --------------------------------------------------------

function EnhancedStackSplit_InitOptions()
	SlashCmdList["ENHANCEDSTACKSPLIT"] = SlashHandler
	SLASH_ENHANCEDSTACKSPLIT1 = "/ess"
	SLASH_ENHANCEDSTACKSPLIT2 = "/enhancedstacksplit"

	EnhancedStackSplit_Options.version = 3
	if EnhancedStackSplit_Options.mode == nil then EnhancedStackSplit_Options.mode = 1     end
	if EnhancedStackSplit_Options.chat == nil then EnhancedStackSplit_Options.chat = false end
end

function EnhancedStackSplit_CreateInterfaceOptions()
	local options = CreateFrame("FRAME", "EnhancedStackSplit_InterfaceOptions")
	options.name = "EnhancedStackSplit"

	local title = options:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetText("EnhancedStackSplit")
	title:SetJustifyH("LEFT")
	title:SetJustifyV("TOP")
	title:SetPoint("TOPLEFT", 16, -16)

	local generalText = options:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	generalText:SetText(L["General"]..":")
	generalText:SetNonSpaceWrap(true)
	generalText:SetJustifyH("LEFT")
	generalText:SetJustifyV("TOP")
	generalText:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -20)

	local chat = CreateFrame("CheckButton", "EnhancedStackSplit_InterfaceOptions_Chat", EnhancedStackSplit_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	EnhancedStackSplit_InterfaceOptions_Chat:SetPoint("TOPLEFT", generalText, "BOTTOMLEFT", 0, -6)
	EnhancedStackSplit_InterfaceOptions_ChatText:SetText(L["Chat Announce"])
	EnhancedStackSplit_InterfaceOptions_Chat:SetChecked(EnhancedStackSplit_Options.chat)
	EnhancedStackSplit_InterfaceOptions_Chat:SetScript("OnClick", function(self)
		EnhancedStackSplit_Options.chat = not EnhancedStackSplit_Options.chat
	end)
	chat.tooltipText = L["Shows a message when the split mode is changed."]

	InterfaceOptions_AddCategory(options)
end

function EnhancedStackSplit_CreateFrames()
	local function createButton(name, parent, template, anchorparent, width, anchorp, anchorrp, adimx, adimy, text, func1, func2)
		local b = CreateFrame("Button", name, parent, template)
		b:SetWidth(width)
		b:SetHeight(24)
		b:SetPoint(anchorp, anchorparent, anchorrp, adimx, adimy)
		b:SetText(text)
		b:SetScript("OnClick", func1)
		if func2 then
			b:SetScript("OnEnter", func2)
		end
		return ceil(b:GetTextWidth())+10
	end

	local f = CreateFrame("Frame", "EnhancedStackSplitTopTextureFrame", StackSplitFrame)
	f:SetPoint("TOPLEFT", "StackSplitFrame", "TOPLEFT", 0, 2)
	f:SetWidth(172)
	f:SetHeight(20)
	local x = f:CreateTexture("EnhancedStackSplitTopTextureFrameTexture", "BACKGROUND")
	x:SetTexture("Interface\\MoneyFrame\\UI-MoneyFrame2")
	x:SetAllPoints(f)
	x:SetTexCoord(0, 172/256, 0, 20/128)
	f.texture = x
	local t = f:CreateFontString("EnhancedStackSplitTextFrameTXT", "BACKGROUND", "GameFontNormalSmall")
	t:SetPoint("TOP", "EnhancedStackSplitTopTextureFrame", "TOP", 1, -8)
	t:SetJustifyH("CENTER")

	local f = CreateFrame("Frame", "EnhancedStackSplitBottomTextureFrame", StackSplitFrame)
	f:SetPoint("TOPLEFT", "StackSplitFrame", "BOTTOMLEFT", 0, 20)
	f:SetWidth(172)
	f:SetHeight(50)
	local x = f:CreateTexture("EnhancedStackSplitBottomTextureFrameTexture", "BACKGROUND")
	x:SetTexture("Interface\\MoneyFrame\\UI-MoneyFrame")
	x:SetAllPoints(f)
	x:SetTexCoord(0, 172/256, 48/128, 96/128)
	f.texture = x

	local f = CreateFrame("Frame", "EnhancedStackSplitAutoTextureFrame", StackSplitFrame)
	f:SetPoint("TOPLEFT", "StackSplitFrame", "TOPLEFT", 16, -13)
	f:SetWidth(142)
	f:SetHeight(37)
	f:Hide()
	local x = f:CreateTexture("EnhancedStackSplitAutoTextureFrameTexture", "BACKGROUND")
	x:SetTexture("Interface\\MoneyFrame\\UI-MoneyFrame2")
	x:SetAllPoints(f)
	x:SetTexCoord(16/256, 158/256, 13/128, 50/128)
	f.texture = x

	local x = f:CreateTexture("EnhancedStackSplitAutoTextureFrameWarningTexture", "HIGH")
	x:SetTexture("Interface\\DialogFrame\\DialogAlertIcon")
	x:SetPoint("BOTTOM", "EnhancedStackSplitAutoTextureFrame", "TOPRIGHT", 6, 0)
	x:SetWidth(41)
	x:SetHeight(32)
	x:SetTexCoord(11/64, 52/64, 16/64, 48/64)
	f.texture = x
	local x = f:CreateTexture("EnhancedStackSplitAutoTextureFrameWarningTexture", "HIGH")
	x:SetTexture("Interface\\DialogFrame\\DialogAlertIcon")
	x:SetPoint("BOTTOM", "EnhancedStackSplitAutoTextureFrame", "TOPLEFT", -6, 0)
	x:SetWidth(41)
	x:SetHeight(32)
	x:SetTexCoord(11/64, 52/64, 16/64, 48/64)
	f.texture = x

	createButton("EnhancedStackSplitAuto1Button",StackSplitFrame,"UIPanelButtonTemplate","StackSplitFrame",64,"RIGHT","BOTTOM",-3,40,"1",function() EnhancedStackSplit_Split(1) end,function() EnhancedStackSplit_AutoSplitInfo(1) end)
	EnhancedStackSplitAuto1Button:Hide()

	local t = f:CreateFontString("EnhancedStackSplitTextAutoText1", "BACKGROUND", "GameFontNormalSmall")
	t:SetPoint("TOP", "EnhancedStackSplitTextFrameTXT", "BOTTOM", 0, -2)
	t:SetJustifyH("CENTER")
	local t = f:CreateFontString("EnhancedStackSplitTextAutoText2", "BACKGROUND", "GameFontNormalSmall")
	t:SetPoint("TOP", "EnhancedStackSplitTextAutoText1", "BOTTOM", 0, -2)
	t:SetJustifyH("CENTER")

	createButton("EnhancedStackSplitButton1" ,EnhancedStackSplitBottomTextureFrame,"UIPanelButtonTemplate2","EnhancedStackSplitBottomTextureFrame",22,"TOPLEFT","TOPLEFT" , 10,  9, "2",function() EnhancedStackSplit_Split(2) end ,function() EnhancedStackSplit_AutoSplitInfo(2) end)
	createButton("EnhancedStackSplitButton2" ,EnhancedStackSplitBottomTextureFrame,"UIPanelButtonTemplate2","EnhancedStackSplitButton1"           ,22,"TOPLEFT","TOPRIGHT", -3,  0, "3",function() EnhancedStackSplit_Split(3) end ,function() EnhancedStackSplit_AutoSplitInfo(3) end)
	createButton("EnhancedStackSplitButton3" ,EnhancedStackSplitBottomTextureFrame,"UIPanelButtonTemplate2","EnhancedStackSplitButton2"           ,22,"TOPLEFT","TOPRIGHT", -3,  0, "4",function() EnhancedStackSplit_Split(4) end ,function() EnhancedStackSplit_AutoSplitInfo(4) end)
	createButton("EnhancedStackSplitButton4" ,EnhancedStackSplitBottomTextureFrame,"UIPanelButtonTemplate2","EnhancedStackSplitButton3"           ,22,"TOPLEFT","TOPRIGHT", -3,  0, "5",function() EnhancedStackSplit_Split(5) end ,function() EnhancedStackSplit_AutoSplitInfo(5) end)
	createButton("EnhancedStackSplitButton5" ,EnhancedStackSplitBottomTextureFrame,"UIPanelButtonTemplate2","EnhancedStackSplitButton4"           ,22,"TOPLEFT","TOPRIGHT", -3,  0, "6",function() EnhancedStackSplit_Split(6) end ,function() EnhancedStackSplit_AutoSplitInfo(6) end)
	createButton("EnhancedStackSplitButton6" ,EnhancedStackSplitBottomTextureFrame,"UIPanelButtonTemplate2","EnhancedStackSplitButton5"           ,22,"TOPLEFT","TOPRIGHT", -3,  0, "7",function() EnhancedStackSplit_Split(7) end ,function() EnhancedStackSplit_AutoSplitInfo(7) end)
	createButton("EnhancedStackSplitButton7" ,EnhancedStackSplitBottomTextureFrame,"UIPanelButtonTemplate2","EnhancedStackSplitButton6"           ,22,"TOPLEFT","TOPRIGHT", -3,  0, "8",function() EnhancedStackSplit_Split(8) end ,function() EnhancedStackSplit_AutoSplitInfo(8) end)
	createButton("EnhancedStackSplitButton8" ,EnhancedStackSplitBottomTextureFrame,"UIPanelButtonTemplate2","EnhancedStackSplitButton7"           ,22,"TOPLEFT","TOPRIGHT", -3,  0, "9",function() EnhancedStackSplit_Split(9) end ,function() EnhancedStackSplit_AutoSplitInfo(9) end)
	createButton("EnhancedStackSplitButton9" ,EnhancedStackSplitBottomTextureFrame,"UIPanelButtonTemplate2","EnhancedStackSplitBottomTextureFrame",26,"TOPLEFT","TOPLEFT" , 30,-12,"10",function() EnhancedStackSplit_Split(10) end,function() EnhancedStackSplit_AutoSplitInfo(10) end)
	createButton("EnhancedStackSplitButton10",EnhancedStackSplitBottomTextureFrame,"UIPanelButtonTemplate2","EnhancedStackSplitButton9"           ,26,"TOPLEFT","TOPRIGHT", -3,  0,"20",function() EnhancedStackSplit_Split(20) end,function() EnhancedStackSplit_AutoSplitInfo(20) end)

	local w = 0
	local wsum = 48 -- width from '10'+'20'-button
	local offset = 10
	w = createButton("EnhancedStackSplitModeTXTButton",EnhancedStackSplitBottomTextureFrame,"UIPanelButtonTemplate2","EnhancedStackSplitButton10",1,"TOPLEFT","TOPRIGHT",offset,0,L["M"],function() EnhancedStackSplit_ModeToggle(1) end)
	EnhancedStackSplitModeTXTButton:SetWidth(w)
	wsum = wsum + w + offset

	w = createButton("EnhancedStackSplitAutoSplitButton",EnhancedStackSplitBottomTextureFrame,"UIPanelButtonTemplate2","EnhancedStackSplitModeTXTButton",1,"TOPLEFT","TOPRIGHT",offset,0,L["Auto"],function() EnhancedStackSplit_ModeToggle(3) end)
	EnhancedStackSplitAutoSplitButton:SetWidth(w)
	wsum = wsum + w + offset
	
	EnhancedStackSplitButton9:SetPoint("TOPLEFT", "EnhancedStackSplitBottomTextureFrame", "TOPLEFT", ceil((172+2-wsum)/2), -12)
	EnhancedStackSplit_ModeSettings(EnhancedStackSplit_Options.mode)
end

function EnhancedStackSplit_ModeToggle(mode)
	if mode == 3 then
		autoSplitMode = true
		Print(string.format(L["StackSplit click mode changed to '%s'."],"|cffffff99"..splitMode[3].."|r").." "..string.format(L["%sATTENTION:%s no OK to confirm!!!"],"|cffff9900","|r"))
		EnhancedStackSplit_ModeSettings(3)
	else
		if not autoSplitMode then
			if EnhancedStackSplit_Options.mode == 2 then
				EnhancedStackSplit_Options.mode = 1
				Print(string.format(L["StackSplit click mode changed to '%s'."],"|cffffff99"..splitMode[1].."|r"))
			else
				EnhancedStackSplit_Options.mode = 2
				Print(string.format(L["StackSplit click mode changed to '%s'."],"|cffffff99"..splitMode[2].."|r").." "..string.format(L["%sATTENTION:%s no OK to confirm!!!"],"|cffff9900","|r"))
			end
		end
		autoSplitMode = false
		EnhancedStackSplit_ModeSettings(EnhancedStackSplit_Options.mode)
	end
end

function EnhancedStackSplit_AutoSplitButtonToggle(toggle)
	if toggle then
		for i=1, 10 do
			EnhancedStackSplit_ButtonTweak("EnhancedStackSplitButton"..i,1)--_G["EnhancedStackSplitButton"..i]:Enable()
		end
		EnhancedStackSplitAuto1Button:Enable()
		EnhancedStackSplitModeTXTButton:Enable()
		StackSplitCancelButton:Enable()
	else
		for i=1, 10 do
			EnhancedStackSplit_ButtonTweak("EnhancedStackSplitButton"..i,0)--_G["EnhancedStackSplitButton"..i]:Disable()
		end
		EnhancedStackSplitAuto1Button:Disable()
		EnhancedStackSplitModeTXTButton:Disable()
		StackSplitCancelButton:Disable()
	end
end

function EnhancedStackSplit_ButtonTweak(button, state)
	-- button: string
	-- state : 0=disable | 1=enable
	if state == 0 then
		_G[button]:Disable()
		_G[button]:SetDisabledTexture(nil)
		_G[button.."Left"]:SetTexture("Interface\\Buttons\\UI-Panel-Button-Disabled")
		_G[button.."Middle"]:SetTexture("Interface\\Buttons\\UI-Panel-Button-Disabled")
		_G[button.."Right"]:SetTexture("Interface\\Buttons\\UI-Panel-Button-Disabled")
		_G[button]:SetScript("OnMouseDown", nil)
		_G[button]:SetScript("OnMouseUp", nil)
	else
		_G[button]:Enable()
		_G[button.."Left"]:SetTexture("Interface\\Buttons\\UI-Panel-Button-Up")
		_G[button.."Middle"]:SetTexture("Interface\\Buttons\\UI-Panel-Button-Up")
		_G[button.."Right"]:SetTexture("Interface\\Buttons\\UI-Panel-Button-Up")
	end
end

function EnhancedStackSplit_ModeSettings(mode)
	for i=1, 10 do
		EnhancedStackSplit_ButtonTweak("EnhancedStackSplitButton"..i,1)--_G["EnhancedStackSplitButton"..i]:Enable()
	end
	if mode == 3 then
		if maxStackSize > 1 and maxStackSize < 11 then
			for i=maxStackSize-1, 10 do
				EnhancedStackSplit_ButtonTweak("EnhancedStackSplitButton"..i,0)--_G["EnhancedStackSplitButton"..i]:Disable()
			end
		end
		if maxStackSize > 10 and maxStackSize < 21 then
			EnhancedStackSplit_ButtonTweak("EnhancedStackSplitButton10",0)--EnhancedStackSplitButton10:Disable()
		end
		EnhancedStackSplit_ButtonTweak("EnhancedStackSplitAutoSplitButton",0)--EnhancedStackSplitAutoSplitButton:Disable()
		StackSplitOkayButton:Hide()
		StackSplitLeftButton:Hide()
		StackSplitRightButton:Hide()
		EnhancedStackSplitAutoTextureFrame:Show()
		EnhancedStackSplitAuto1Button:Show()
		EnhancedStackSplitTextAutoText1:Show()
		EnhancedStackSplitTextAutoText2:Show()
		EnhancedStackSplitTextAutoText1:SetText("|cffffffff"..L["Free Bag Slots"]..":|r |cffff8000"..GetFreeBagSlots().."|r")
		EnhancedStackSplitTextAutoText2:SetText("")
	else
		if maxStackSize > 1 and maxStackSize < 11 then
			for i=maxStackSize, 10 do
				EnhancedStackSplit_ButtonTweak("EnhancedStackSplitButton"..i,0)--_G["EnhancedStackSplitButton"..i]:Disable()
			end
		end
		if not autoSplitItemLink then
			EnhancedStackSplit_ButtonTweak("EnhancedStackSplitAutoSplitButton",0)--EnhancedStackSplitAutoSplitButton:Disable()
		else
			EnhancedStackSplit_ButtonTweak("EnhancedStackSplitAutoSplitButton",1)--EnhancedStackSplitAutoSplitButton:Enable()
		end
		EnhancedStackSplitAutoTextureFrame:Hide()
		EnhancedStackSplitAuto1Button:Hide()
		EnhancedStackSplitTextAutoText1:Hide()
		EnhancedStackSplitTextAutoText2:Hide()
		StackSplitOkayButton:Show()
		StackSplitLeftButton:Show()
		StackSplitRightButton:Show()
	end
	EnhancedStackSplitTextFrameTXT:SetText(splitMode[mode])
end

function EnhancedStackSplit_AutoSplitInfo(num)
	local freeSlots, numStacks, leftOver = AutoSplitCalc(num)
	EnhancedStackSplitTextAutoText2:SetText("|cffffffff"..maxStackSize.." | |cffff8000"..numStacks.."|cffffffffx "..num.." | "..L["leftover"].."="..leftOver.."|r")
end

function EnhancedStackSplit_OpenStackSplitFrame(maxStack, parent, anchor, anchorTo)
	if maxStack < 2 then return end

	splitItemBag = StackSplitFrame.owner:GetParent():GetID()
	splitItemSlot = StackSplitFrame.owner:GetID()

	local splitItemName = StackSplitFrame.owner:GetParent():GetName()
	if splitItemName then
		for i = 1, #mainBankFrames do -- this needs some better solution
			if string.find(splitItemName, mainBankFrames[i]) then
				splitItemBag = -1
			end
		end
		for i = 1, #blockedFrames do -- this needs some better solution
			if string.find(splitItemName, blockedFrames[i]) then
				splitItemBag = nil
			end
		end
	end

	autoSplitItemLink = nil
	if splitItemBag then
		autoSplitItemLink = GetContainerItemLink(splitItemBag, splitItemSlot)
	end

	maxStackSize = maxStack
	StackSplitFrame:SetPoint(anchor, parent, anchorTo, 0, 14)
	StackSplitOkayButton:SetPoint("RIGHT", "StackSplitFrame", "BOTTOM", -3, 40)
	StackSplitCancelButton:SetPoint("LEFT", "StackSplitFrame", "BOTTOM", 5, 40)
	autoSplitMode = false
	EnhancedStackSplit_AutoSplitButtonToggle(true)
	EnhancedStackSplitTextFrameTXT:SetText(splitMode[EnhancedStackSplit_Options.mode])
	EnhancedStackSplit_ModeSettings(EnhancedStackSplit_Options.mode)
end

function EnhancedStackSplit_Split(num)
	if autoSplitMode then
		local freeSlots, numStacks, leftOver = AutoSplitCalc(num)
		autoSplitStackCount = 0
		autoSplitNumStacks = numStacks
		autoSplitLeftOver = leftOver
		autoSplitNumber = num
		ClearSlotLocks()
		EnhancedStackSplit_AutoSplitButtonToggle(false)
		EnhancedStackSplit_AutoSplit(autoSplitNumStacks, autoSplitNumber, autoSplitLeftOver)
	else
		EnhancedStackSplit_SetNewSplitSize(num)
	end
end

function EnhancedStackSplit_CheckItemLock()
	if StackSplitFrame:IsShown() and StackSplitFrame.owner and splitItemBag and splitItemSlot then
		if arg1 and arg2 then
			if arg1 == splitItemBag and arg2 == splitItemSlot then
				local locked = select(3, GetContainerItemInfo(splitItemBag, splitItemSlot))
				if not locked then
					EnhancedStackSplit_AutoSplit(autoSplitNumStacks, autoSplitNumber, autoSplitLeftOver)
				end
			end
		end
	end
end

function EnhancedStackSplit_AutoSplit(numStacks, num, leftOver)
	if leftOver == 0 then
		if autoSplitStackCount == numStacks-1 then
			StackSplitFrame:Hide() return
		end
	else
		if autoSplitStackCount == numStacks then
			StackSplitFrame:Hide() return
		end
	end
	local bag, slot = GetFreeSlot()
	SplitContainerItem(splitItemBag, splitItemSlot, num)
	if bag ~= nil then
		LockSlot(bag, slot)
		PickupContainerItem(bag, slot)
	end
	autoSplitStackCount = autoSplitStackCount + 1
end

function EnhancedStackSplit_SetNewSplitSize(num)
	if num >= StackSplitFrame.maxStack then
		num = StackSplitFrame.maxStack
		StackSplitRightButton:Disable()
	end
	if num < StackSplitFrame.maxStack then
		StackSplitRightButton:Enable()
	end
	StackSplitLeftButton:Enable()
	StackSplitFrame.split = num
	StackSplitText:SetText(num)
	if EnhancedStackSplit_Options.mode == 2 then
		StackSplitFrameOkay_Click()
	end
end

function EnhancedStackSplit_OnEvent()
	if event == "ITEM_LOCK_CHANGED" then
		EnhancedStackSplit_CheckItemLock()
	elseif event == "ADDON_LOADED" then
		if arg1 == "EnhancedStackSplit" then
			EnhancedStackSplit_InitOptions()
			EnhancedStackSplit_CreateInterfaceOptions()
			EnhancedStackSplit_CreateFrames()
			hooksecurefunc("OpenStackSplitFrame", EnhancedStackSplit_OpenStackSplitFrame)
			EnhancedStackSplit:UnregisterEvent("ADDON_LOADED")
		end
	end
end

EnhancedStackSplit:RegisterEvent("ADDON_LOADED")
EnhancedStackSplit:RegisterEvent("ITEM_LOCK_CHANGED")
EnhancedStackSplit:SetScript("OnEvent", EnhancedStackSplit_OnEvent)